@@ -1,3 +1,5 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* Metacity X screen handler */
 
 /* 
@@ -52,7 +54,8 @@
 static char* get_screen_name (MetaDisplay *display,
                               int          number);
 
-static void update_num_workspaces  (MetaScreen *screen);
+static void update_num_workspaces  (MetaScreen *screen,
+                                    guint32     timestamp);
 static void update_focus_mode      (MetaScreen *screen);
 static void set_workspace_names    (MetaScreen *screen);
 static void prefs_changed_callback (MetaPreference pref,
@@ -187,6 +190,20 @@ reload_xinerama_infos (MetaScreen *screen)
 {
   MetaDisplay *display;
 
+  {
+    GList *tmp;
+
+    tmp = screen->workspaces;
+    while (tmp != NULL)
+      {
+        MetaWorkspace *space = tmp->data;
+
+        meta_workspace_invalidate_work_area (space);
+        
+        tmp = tmp->next;
+      }
+  }
+
   display = screen->display;
   
   if (screen->xinerama_infos)
@@ -348,26 +365,12 @@ reload_xinerama_infos (MetaScreen *screen)
 
   g_assert (screen->n_xinerama_infos > 0);
   g_assert (screen->xinerama_infos != NULL);
-
-  {
-    GList *tmp;
-
-    tmp = screen->workspaces;
-    while (tmp != NULL)
-      {
-        MetaWorkspace *space = tmp->data;
-
-        meta_workspace_invalidate_work_area (space);
-        
-        tmp = tmp->next;
-      }
-  }
 }
 
 MetaScreen*
 meta_screen_new (MetaDisplay *display,
                  int          number,
-                 Time         timestamp)
+                 guint32      timestamp)
 {
   MetaScreen *screen;
   Window xroot;
@@ -378,7 +381,7 @@ meta_screen_new (MetaDisplay *display,
   gboolean replace_current_wm;
   Atom wm_sn_atom;
   char buf[128];
-  Time manager_timestamp;
+  guint32 manager_timestamp;
   gulong current_workspace;
   
   replace_current_wm = meta_get_replace_current_wm ();
@@ -431,7 +434,10 @@ meta_screen_new (MetaDisplay *display,
         current_wm_sn_owner = None; /* don't wait for it to die later on */
     }
 
-  new_wm_sn_owner = meta_create_offscreen_window (xdisplay, xroot);
+  /* We need SelectionClear and SelectionRequest events on the new_wm_sn_owner,
+   * but those cannot be masked, so we only need NoEventMask.
+   */
+  new_wm_sn_owner = meta_create_offscreen_window (xdisplay, xroot, NoEventMask);
 
   manager_timestamp = timestamp;
   
@@ -574,10 +580,10 @@ meta_screen_new (MetaDisplay *display,
   meta_screen_set_cursor (screen, META_CURSOR_DEFAULT);
 
   /* Handle creating a no_focus_window for this screen */  
-  screen->no_focus_window = meta_create_offscreen_window (display->xdisplay,
-                                                          screen->xroot);
-  XSelectInput (display->xdisplay, screen->no_focus_window,
-                FocusChangeMask | KeyPressMask | KeyReleaseMask);
+  screen->no_focus_window =
+    meta_create_offscreen_window (display->xdisplay,
+                                  screen->xroot,
+                                  FocusChangeMask|KeyPressMask|KeyReleaseMask);
   XMapWindow (display->xdisplay, screen->no_focus_window);
   /* Done with no_focus_window stuff */
   
@@ -608,7 +614,7 @@ meta_screen_new (MetaDisplay *display,
    * so create that required workspace.
    */
   meta_workspace_activate (meta_workspace_new (screen), timestamp);
-  update_num_workspaces (screen);
+  update_num_workspaces (screen, timestamp);
   
   set_workspace_names (screen);
 
@@ -654,7 +660,8 @@ meta_screen_new (MetaDisplay *display,
 }
 
 void
-meta_screen_free (MetaScreen *screen)
+meta_screen_free (MetaScreen *screen,
+                  guint32     timestamp)
 {
   MetaDisplay *display;
   XGCValues gc_values = { 0 };
@@ -671,7 +678,7 @@ meta_screen_free (MetaScreen *screen)
 				       screen);
     }
   
-  meta_display_unmanage_windows_for_screen (display, screen);
+  meta_display_unmanage_windows_for_screen (display, screen, timestamp);
   
   meta_prefs_remove_listener (prefs_changed_callback, screen);
   
@@ -855,7 +862,12 @@ prefs_changed_callback (MetaPreference pref,
   
   if (pref == META_PREF_NUM_WORKSPACES)
     {
-      update_num_workspaces (screen);
+      /* GConf doesn't provide timestamps, but luckily update_num_workspaces
+       * often doesn't need it...
+       */
+      guint32 timestamp = 
+        meta_display_get_current_time_roundtrip (screen->display);
+      update_num_workspaces (screen, timestamp);
     }
   else if (pref == META_PREF_FOCUS_MODE)
     {
@@ -1083,7 +1095,8 @@ set_desktop_viewport_hint (MetaScreen *screen)
 }
 
 static void
-update_num_workspaces (MetaScreen *screen)
+update_num_workspaces (MetaScreen *screen,
+                       guint32     timestamp)
 {
   int new_num;
   GList *tmp;
@@ -1136,7 +1149,7 @@ update_num_workspaces (MetaScreen *screen)
     }
 
   if (need_change_space)
-    meta_workspace_activate (last_remaining, meta_display_get_current_time_roundtrip (screen->display));
+    meta_workspace_activate (last_remaining, timestamp);
 
   /* Should now be safe to free the workspaces */
   tmp = extras;
@@ -1805,7 +1818,8 @@ meta_screen_update_workspace_names (MetaScreen *screen)
 
 Window
 meta_create_offscreen_window (Display *xdisplay,
-                              Window   parent)
+                              Window   parent,
+                              long     valuemask)
 {
   XSetWindowAttributes attrs;
 
@@ -1814,7 +1828,7 @@ meta_create_offscreen_window (Display *xdisplay,
    * (but on a display we are managing at least one screen for)
    */
   attrs.override_redirect = True;
-  attrs.event_mask = PropertyChangeMask;
+  attrs.event_mask = valuemask;
   
   return XCreateWindow (xdisplay,
                         parent,
@@ -2202,6 +2216,8 @@ meta_screen_resize_func (MetaScreen *screen,
       meta_window_update_struts (window);
     }
   meta_window_queue_move_resize (window);
+
+  meta_window_recalc_features (window);
 }
 
 void
@@ -2285,7 +2301,7 @@ meta_screen_minimize_all_on_active_workspace_except (MetaScreen *screen,
 
 void
 meta_screen_show_desktop (MetaScreen *screen, 
-                          Time        timestamp)
+                          guint32     timestamp)
 {
   GList *windows;
 
@@ -2602,7 +2618,7 @@ meta_screen_apply_startup_properties (MetaScreen *screen,
   if (sequence != NULL)
     {
       int space;
-      Time timestamp;
+      guint32 timestamp;
           
       meta_topic (META_DEBUG_STARTUP,
                   "Found startup sequence for window %s ID \"%s\"\n",
@@ -2626,7 +2642,7 @@ meta_screen_apply_startup_properties (MetaScreen *screen,
         {
           timestamp = sn_startup_sequence_get_timestamp (sequence);
           meta_topic (META_DEBUG_STARTUP,
-                      "Setting initial window timestamp to %lu based on startup info\n",
+                      "Setting initial window timestamp to %u based on startup info\n",
                       timestamp);
               
           window->initial_timestamp_set = TRUE;
